var Protocol = function()
{
	var protocol = {};
	var debug = 0;
	
	var LOW_QUANTITY_WARNING_FACTOR = 2;
	var DEFAULT_DILUTION_VOLUME = 50; //µl

	// Page initialization
	protocol.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'protocol')
		{
			Buttons.addClickHandler('print-button', Wizard.goPrint);	

			var plateId = Data.int('page-data', 'plate-id');

			var url = '../LibPrep.servlet?ID='+App.getSessionId();
			url += '&cmd=GetLibraryInfoForPlate&bioplate='+plateId;		
			url += '&loadDilutionInfo=1';
			Wizard.showLoadingAnimation('Loading library plate information...');
			Wizard.asyncJsonRequest(url, protocol.initializeProtocol);
		}
		else
		{
			Events.addEventHandler('bioplate', 'change', protocol.bioplateOnChange);
			Events.addEventHandler('listview', 'click', protocol.viewProtocol);
			Events.addEventHandler('plateview', 'click', protocol.viewProtocol);
			Events.addEventHandler('downloadLabelsCsv', 'click', protocol.downloadLabels);
			Events.addEventHandler('downloadLabelsXlsx', 'click', protocol.downloadLabels);
			Events.addEventHandler('ctgRnaSampleSheet', 'click', protocol.downloadSampleSheet);

			var url = '../LibPrep.servlet?ID='+App.getSessionId();
			url += '&cmd=GetLibraryPlatesForLibPrep&plateType=EXTERNAL_LIBRARY';
			Wizard.showLoadingAnimation('Loading library plates...');
			Wizard.asyncJsonRequest(url, protocol.initializeStep1);
		}
	}


	protocol.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];	

		var bioplates = response.bioplates;
		var plates = frm.bioplate;
		if (bioplates.length > 0)
		{
			for (var i=0; i < bioplates.length; i++)
			{
				var bioplate = bioplates[i];
				var name = bioplate.name + ' (' + bioplate.usedWells + ' samples';
				if (bioplate.AutoProcessing == 'PreNormalizeRNA')
				{
					name += '; pre-normalized';
				}
				else if (bioplate.DilutionDate)
				{
					name += '; diluted ' + Reggie.reformatDate(bioplate.DilutionDate);
				}
				name += ')';
				var option = new Option(name, bioplate.id);
				option.bioplate = bioplate;
				plates.options[plates.length] = option;
			}
			bioplateIsValid = true;
			
			protocol.bioplateOnChange();
		}
		else
		{
			Wizard.setFatalError('No library plates available for processing.');
			return;
		}

		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	
	// Add pools to the pools list based on the bioplate selection
	protocol.bioplateOnChange = function()
	{
		var frm = document.forms['reggie'];
		var bioplate = frm.bioplate[frm.bioplate.selectedIndex].bioplate;
		Doc.element('comments').innerHTML = Strings.encodeTags(bioplate.comments);
		frm.poolSchema.value = bioplate.poolSchema;
	}
	
	protocol.viewProtocol = function(event)
	{
		var frm = document.forms['reggie'];
		if (frm.bioplate && !frm.bioplate.disabled)
		{
			frm.view.value = Data.get(event.currentTarget, 'protocol-type');
			frm.submit();
		}
	}
	
	protocol.downloadLabels = function(event)
	{	
		var frm = document.forms['reggie'];
		
		var url = '../LibPrep.servlet?ID='+App.getSessionId();
		url += '&cmd=DownloadLabelsForExternalSequencing';
		url += '&bioPlateId='+frm.bioplate.value;
		url += '&includeAll='+(frm.includeAll.checked ? 1 : 0);
		url += '&format='+encodeURIComponent(Data.get(event.currentTarget, 'format'));

		window.open(url);
	}

	protocol.downloadSampleSheet = function(event)
	{
		var frm = document.forms['reggie'];
		var url = '../LibPrep.servlet?ID='+App.getSessionId();
		url += '&cmd=GetExternalSequencingSampleSheet';
		url += '&bioPlateId='+frm.bioplate.value;
		url += '&template='+encodeURIComponent(Data.get(event.currentTarget, 'template'));
		window.open(url);
	}

	protocol.initializeProtocol = function(response)
	{
		var plate = response.plate;
		var list = response.libraries;
		var normProtocol = response.protocol;

		var unit = 'µg';
		var factor = 1;
		if (normProtocol.RNATargetAmount <= 0.5)
		{
			unit = 'ng';
			factor = 1000;
		}
		
		var text = Strings.encodeTags(normProtocol.name) + ' (';
		text += normProtocol.RNATargetAmount ? Reggie.formatNumber(normProtocol.RNATargetAmount * factor, '', 1) : '? ';
		text += unit;
		text += ' in '+(normProtocol.RNATargetVolume || '? ')+'µl)';
		Doc.element('normalization-protocol').innerHTML = text;
		
		// Pre-process the return mRNA items
		var allPreNormalized = true;
		for (var i = 0; i < list.length; i++)
		{
			allPreNormalized &= list[i].rna.preNormalized;
			protocol.checkAndPreProcessLib(list[i]);
		}
		
		var view = Data.get('page-data', 'view');
		if (view == 'list')
		{
			protocol.viewAsList(list);
		}
		else
		{
			protocol.viewAsPlate(plate, list)
		}
		
	}
	
	protocol.checkAndPreProcessLib = function(lib, stratageneConc)
	{
		var rna = lib.rna;
		// Set the 'stratagene' flag
		rna.stratagene = Reggie.isStratagene(rna.name);
		rna.external = Reggie.isExternal(rna.name);
		rna.isYellow = rna.specimen && rna.specimen.YellowLabel != null;
		if (!lib.DilutionVolume) lib.DilutionVolume = DEFAULT_DILUTION_VOLUME;
		
		// Set the 'QC' flag 
		rna.qc = lib.UseForQC;
		
		if (stratageneConc > 0)
		{
			if (rna.stratagene && !rna.conc) rna.conc = stratageneConc;
		}
		
		// Calculate volume to use and water to add based on concentration
		var remarks = [];
		if (rna.conc && rna.usedQuantity)
		{
			rna.volume = Math.ceil(10000*rna.usedQuantity/rna.conc) / 10; // µl
			
			var totalVolume = lib.DilutionConc ? 1000 * rna.usedQuantity / lib.DilutionConc : lib.DilutionVolume;
			if (totalVolume < lib.DilutionVolume) 
			{
				totalVolume = lib.DilutionVolume;
			}
			else if (totalVolume - 0.05 > lib.DilutionVolume) // Use 0.05 to avoid issues with 50.0000000001 > 50.0
			{
				remarks[remarks.length] = 'Large mix; Use '+Reggie.formatNumber(lib.DilutionVolume, 'µl', 1);
			}
			rna.water = totalVolume-rna.volume;
		}
		
		// check for some issues that may indicate problems
		if (rna.qc) remarks[remarks.length] = 'QC';
		if (!rna.stratagene && !rna.external)
		{
			if (!rna.bioWell && !rna.preNormalized)
			{
				remarks[remarks.length] = 'No location';
			}
			if (!rna.remainingQuantity)
			{
				remarks[remarks.length] = 'No quantity';
			}
			else if (rna.remainingQuantity < rna.usedQuantity)
			{
				remarks[remarks.length] = 'Not enough RNA';
			}
			else if (rna.remainingQuantity < rna.usedQuantity * LOW_QUANTITY_WARNING_FACTOR && !rna.preNormalized)
			{
				remarks[remarks.length] = 'Low quantity';
			}
			if (!rna.conc)
			{
				remarks[remarks.length] = 'No concentration';
			}
		}
		else
		{
			if (!rna.conc && rna.usedQuantity) 
			{
				remarks[remarks.length] = 'Use ' + Numbers.formatNumber(rna.usedQuantity, 2) + ' µg RNA';
			}
		}
		if (lib.comment) remarks[remarks.length] = lib.comment;
		rna.remarks = remarks;
	}

	protocol.viewAsPlate = function(plate, list)
	{
		var schema = PoolSchema.getById(plate.poolSchema);
		Plate.init(plate.rows, plate.columns, schema, WellPainter);
		
		for (var i = 0; i < list.length; i++)
		{
			var lib = list[i];
			var well = lib.bioWell;
			Plate.getWell(well.row, well.column).setExtract(lib);
		}

		Plate.paint(Plate.getWells());
		PoolSchema.buildPoolTableRow(schema, plate.columns);
		Doc.show('all-protocol');
	}

	protocol.viewAsList = function(list)
	{
		var homeUrl = Data.get('page-data', 'home-url');
		var yellowImg = '<img src="'+homeUrl+'/images/yellow-label-small.png">';
		for (var i = 0; i < list.length; i++)
		{
			var lib = list[i];
			var rna = lib.rna;
			var idSuffix = lib.bioWell.column + '.' + lib.bioWell.row;
			var tableRow = Doc.element('row.'+idSuffix);
			var img = rna.isYellow ? yellowImg : '';
			
			Doc.removeClass(tableRow, 'empty');
			if (rna.external) Doc.addClass(tableRow, 'external');
			if (rna.stratagene) Doc.addClass(tableRow, 'stratagene');
			if (rna.isYellow) Doc.addClass(tableRow, 'yellow-specimen');
			Doc.element('rna.'+idSuffix).innerHTML = img+Strings.encodeTags(rna.external && lib.externalId ? lib.externalId : rna.name);
			Doc.element('box.'+idSuffix).innerHTML = Strings.encodeTags(protocol.getPlateCoordinate(rna, true));
			Doc.element('conc.'+idSuffix).innerHTML = Numbers.formatNumber(rna.conc, 2);
			Doc.element('remain.'+idSuffix).innerHTML = Numbers.formatNumber(rna.remainingQuantity, 2);
			Doc.element('volume.'+idSuffix).innerHTML = Numbers.formatNumber(rna.volume, 1);
			Doc.element('water.'+idSuffix).innerHTML = rna.water > 0 ? Numbers.formatNumber(rna.water, 1) : '—';
			Doc.element('remarks.'+idSuffix).innerHTML = rna.remarks.join('; ');
		}
		
		Doc.show('all-protocol');
	}
	
	
	protocol.getPlateCoordinate = function(rna, includePlateName)
	{
		var c = '';
		if (rna.bioWell)
		{
			if (includePlateName) c += rna.bioWell.bioPlate.name;
			c += ' ' + Reggie.wellToAlpha(rna.bioWell.row) + (rna.bioWell.column+1);
		}
		else if (rna.preNormalized)
		{
			c = 'PreNorm.';
		}
		else
		{
			c = ' ';
		}
		return c;
	}

		
	return protocol;
}();

Doc.onLoad(Protocol.initPage);


var WellPainter = function()
{
	var painter = {};
	
	painter.getClassNameForWell = function(well)
	{
		var cls = '';
		var lib = well.extract;
		if (lib)
		{
			var rna = lib.rna;
			if (rna.qc) cls += ' qc';
			if (rna.external) cls += ' external';
			if (rna.stratagene) cls += ' stratagene';
			if (rna.isYellow) cls += ' yellow-specimen';
		}
		return cls;
	}
	
	painter.getWellText = function(well)
	{
		var text = '';
		var lib = well.extract;
		if (lib)
		{
			var rna = lib.rna;
			var name = rna.external && lib.externalId ? lib.externalId : rna.name;
			text += '<div class="rna if-yellow">'+name+'</div>';
			text += '<div class="box">'+Strings.encodeTags(Protocol.getPlateCoordinate(rna, true))+'</div>';
			text += '<div class="remain">'+Numbers.formatNumber(rna.remainingQuantity, 2, 'µg')+'</div>';
			text += '<div class="conc">'+Numbers.formatNumber(rna.conc, 2, 'ng/µl') + '</div>';

			if (rna.volume >= 0 && rna.water >= 0)
			{
				text += '<div><span class="volume">'+Numbers.formatNumber(rna.volume, 1)+'µl</span>';
				text += '<span class="water">'+(rna.water > 0 ? Numbers.formatNumber(rna.water, 1)+'µl' : '—')+'</span></div>';
			}
			text += '<div class="remarks">'+ rna.remarks.join('; ') + '</div>';
		}
		return text;
	}

	return painter;
}();

