var Protocol = function()
{
	var protocol = {};
	var debug = 0;
	
	// Page initialization
	protocol.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'protocol')
		{
			Buttons.addClickHandler('print-button', Wizard.goPrint);	

			var mipsPlateId = Data.int('page-data', 'mips-plate');

			var url = '../Mips.servlet?ID='+App.getSessionId();
			url += '&cmd=GetLibraryInfoForPlate&bioplate='+mipsPlateId;		
			Wizard.showLoadingAnimation('Loading MIPs plate information...');
			Wizard.asyncJsonRequest(url, protocol.initializeProtocol);
		}
		else
		{
			Events.addEventHandler('bioplate', 'change', protocol.bioplateOnChange);
			Events.addEventHandler('listview', 'click', protocol.viewProtocol);
			Events.addEventHandler('plateview', 'click', protocol.viewProtocol);

			var url = '../Mips.servlet?ID='+App.getSessionId();
			url += '&cmd=GetUnprocessedPlates&plateType=MIPS';
			Wizard.showLoadingAnimation('Loading MIPs plates...');
			Wizard.asyncJsonRequest(url, protocol.initializeStep1);
		}
	}


	protocol.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];	

		var bioplates = response.bioplates;
		var plates = frm.bioplate;
		if (bioplates.length > 0)
		{
			for (var i=0; i < bioplates.length; i++)
			{
				var bioplate = bioplates[i];
				var name = bioplate.name;
				var option = new Option(name, bioplate.id);
				option.bioplate = bioplate;
				plates.options[plates.length] = option;
			}
			bioplateIsValid = true;
			protocol.bioplateOnChange();
		}
		else
		{
			Wizard.setFatalError('No MIPs plates available for processing.');
			return;
		}

		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	
	// Add pools to the pools list based on the bioplate selection
	protocol.bioplateOnChange = function()
	{
		var frm = document.forms['reggie'];
		var bioplate = frm.bioplate[frm.bioplate.selectedIndex].bioplate;
		Doc.element('comments').innerHTML = Strings.encodeTags(bioplate.comments);
	}
	
	protocol.viewProtocol = function(event)
	{
		var frm = document.forms['reggie'];
		if (frm.bioplate && !frm.bioplate.disabled)
		{
			frm.view.value = Data.get(event.currentTarget, 'protocol-type');
			frm.submit();
		}
	}
	
	protocol.initializeProtocol = function(response)
	{
		var plate = response.plate;
		var list = response.libraries;
		
		protocol.preProcessPlate(plate, list);
		var view = Data.get('page-data', 'view');
		if (view == 'list')
		{
			protocol.viewAsList(list);
		}
		else
		{
			protocol.viewAsPlate(plate, list)
		}
	}
	
	// We need to make column-wize checks on the data
	// For example a single MIPS_Panel is expected per column otherwise we must show a warning
	protocol.preProcessPlate = function(plate, list)
	{
		Plate.init(plate.rows, plate.columns, WellPainter);
		// Place all libraries on the plate and initialize some information
		for (var i = 0; i < list.length; i++)
		{
			var lib = list[i];
			var well = lib.bioWell;
			var dna = lib.dna;
			dna.remarks = [];
			dna.warnings = [];
			if (lib.comment) dna.remarks[dna.remarks.length] = lib.comment;
			Plate.getWell(well.row, well.column).setExtract(lib);
		}
		
		// Analyze columns on the MIPs plate with resepect to MIPS_Panel and PoolVolume
		COLUMN_DATA = new Array(plate.columns);
		for (var colNo = 0; colNo < plate.columns; colNo++)
		{
			var col = {};
			COLUMN_DATA[colNo] = col;
			col.numAliquots = 0;
			col.panelCounter = new ItemCounter();
			col.poolVolumeCounter = new ItemCounter();
			for (var rowNo = 0; rowNo < Plate.rows; rowNo++)
			{
				var well = Plate.getWell(rowNo, colNo);
				if (well.extract)
				{
					var aliquot = well.extract.dna;
					col.numAliquots++;
					col.panelCounter.count(aliquot.MIPS_Panel);
					col.poolVolumeCounter.count(aliquot.MIPS_PoolVolume);
					if (!col.plate && aliquot.bioWell)
					{
						col.plate = aliquot.bioWell.bioPlate.name;
					}
				}
			}
			col.MIPS_Panel = col.panelCounter.maxKey;
			col.isMissingMIPSPanel = col.panelCounter.numCounted != col.numAliquots;
			col.hasMultipleMIPSPanels = col.panelCounter.numKeys() > 1;
			col.MIPS_PoolVolume = col.poolVolumeCounter.maxKey;
			col.hasMultiplePoolVolumes = col.poolVolumeCounter.numKeys() > 1;
		}
		
		// Finally, check library data against column data and generate warnings
		for (var i = 0; i < list.length; i++)
		{
			var lib = list[i];
			var well = lib.bioWell;
			var dna = lib.dna;
			
			if (dna.MIPS_PoolVolume != null)
			{
				if (dna.MIPS_PoolVolume != COLUMN_DATA[well.column].MIPS_PoolVolume)
				{
					dna.warnings[dna.warnings.length] = 'PoolVolume!';
				}
			}
		}
	}
	
	protocol.viewAsPlate = function()
	{
		protocol.buildPoolRow();
		protocol.buildPlateRow();
		Plate.paint(Plate.getWells());
		Doc.show('all-protocol');
	}
	
	protocol.buildPoolRow = function()
	{
		var html = '<th></th>';
		var pools = [];	// A 'pool' is a number of columns with the same MIPS_Panel
		var currentPool = {};

		var poolNum = -1;
		for (var colNo = 0; colNo < COLUMN_DATA.length; colNo++)
		{
			var col = COLUMN_DATA[colNo];
			if ((col.MIPS_Panel != currentPool.name) || (col.numAliquots == 0 && currentPool.numAliquots != 0))
			{
				// The column has a different MIPS_Panel than the previous one so we create a new pool
				currentPool = {};
				currentPool.name = col.MIPS_Panel;
				currentPool.startCol = colNo;
				currentPool.columnSpan = 1;
				currentPool.hasMultipleMIPSPanels = col.hasMultipleMIPSPanels;
				currentPool.hasMultiplePoolVolumes = col.hasMultiplePoolVolumes;
				currentPool.isMissingMIPSPanel = col.isMissingMIPSPanel;
				currentPool.numAliquots = col.numAliquots;
				pools[pools.length] = currentPool;
				poolNum++;
			}
			else if (currentPool)
			{
				// The column has the same MIPS_Panel as the previous one, we make the pool span across multiple columns
				currentPool.columnSpan++;
				currentPool.numAliquots += col.numAliquots;
				if (col.isMissingMIPSPanel) currentPool.isMissingMIPSPanel = true;
				if (col.hasMultipleMIPSPanels) currentPool.hasMultipleMIPSPanels = true;
				if (col.hasMultiplePoolVolumes) currentPool.hasMultiplePoolVolumes = true;
			}
		}
		
		colNo = 0;
		for (var poolNo = 0; poolNo < pools.length; poolNo++)
		{
			var pool = pools[poolNo];
			var poolClass = 'pool link';
			var poolText = null;
			
			if (pool.columnSpan == 1)
			{
				COLUMN_DATA[colNo].poolClass = 'pool-left pool-right';
			}
			else
			{
				COLUMN_DATA[colNo].poolClass = 'pool-left';
				for (var i = 1; i < pool.columnSpan-1; i++)
				{
					COLUMN_DATA[colNo+i].poolClass = 'pool-middle';
				}
				COLUMN_DATA[colNo+pool.columnSpan-1].poolClass = 'pool-right';
			}

			if (pool.numAliquots > 0)
			{
				if (pool.hasMultiplePoolVolumes || pool.isMissingMIPSPanel) 
				{
					poolClass += ' pool-warning';
					if (pool.isMissingMIPSPanel)
					{
						poolText = 'Some aliquots have no MIPS_Panel!';
					}
					else if (pool.hasMultiplePoolVolumes)
					{
						poolText = 'Some aliquots have a different pool volume!';
					}
				}
				if (pool.name) poolClass += ' ' + MipsColor.getColor(pool.name);
				html += '<th id="pool.' + poolNo + '" colspan="'+pool.columnSpan+'"';
				html += ' class="'+poolClass+'"';
				html += ' data-start-col="'+pool.startCol+'"';
				html += ' data-end-col="'+(pool.startCol+pool.columnSpan-1)+'"';
				if (poolText) html += ' title="'+poolText+'"';
				html += '>'+Strings.encodeTags(pool.name || 'No MIPS_Panel')+'</th>';
			}
			else
			{
				poolClass = '';
				if (colNo > 0) poolClass += ' nopool-after-pool';
				if (colNo + pool.columnSpan < 12) poolClass += ' nopool-before-pool';
				html += '<th colspan="'+pool.columnSpan+'"';
				html += ' class="'+poolClass+'"';
				html += '></th>';
			}
			colNo += pool.columnSpan;
		}
		
		var row = Doc.element('pool-row');
		row.innerHTML = html;
		Doc.show(row);
	}
	
	protocol.buildPlateRow = function()
	{
		var html = '<th></th>';
		var poolClass = 'group link';
		var totalSpan = 0;
		var colNo = 0;
		
		var plates = [];
		var currentPlate = {};
		for (var colNo = 0; colNo < COLUMN_DATA.length; colNo++)
		{
			var col = COLUMN_DATA[colNo];
			if ((col.plate != currentPlate.name))
			{
				// The column has a different aliquot plate than the previous one so we create a new pool
				currentPlate = {};
				currentPlate.name = col.plate;
				currentPlate.startCol = colNo;
				currentPlate.columnSpan = 1;
				plates[plates.length] = currentPlate;
			}
			else if (currentPlate)
			{
				// The column has the same aliquot plate as the previous one, we make the pool span across multiple columns
				currentPlate.columnSpan++;
			}
		}

		for (var plateNo = 0; plateNo < plates.length; plateNo++)
		{
			var plate = plates[plateNo];
			if (plate.columnSpan == 1)
			{
				COLUMN_DATA[plate.startCol].groupClass = 'group-left group-right';
			}
			else
			{
				COLUMN_DATA[plate.startCol].groupClass = 'group-left';
				for (var i = 1; i < plate.columnSpan-1; i++)
				{
					COLUMN_DATA[plate.startCol+i].groupClass = 'group-middle';
				}
				COLUMN_DATA[plate.startCol+plate.columnSpan-1].groupClass = 'group-right';
			}
			
			if (plate.name)
			{
				totalSpan += plate.columnSpan;
				html += '<th id="group.' + plateNo + '" colspan="'+plate.columnSpan+'"';
				html += ' class="'+poolClass+'"';
				html += ' data-start-col="' + plate.startCol + '"';
				html += ' data-end-col="' + (plate.startCol+plate.columnSpan-1) + '"';
				html += '>'+Strings.encodeTags(plate.name)+'</th>';
			}
			else
			{
				totalSpan += plate.columnSpan;
				html += '<th class="nopool-after-pool" colspan="'+plate.columnSpan+'"></th>';
			}
		}
		var row = document.getElementById('plate-row');
		row.innerHTML = html;
		Doc.show(row);
	}

	
	protocol.viewAsList = function(list)
	{
		var homeUrl = Data.get('page-data', 'home-url');
		var yellowImg = '<img src="'+homeUrl+'/images/yellow-label-small.png">';
		for (var i = 0; i < list.length; i++)
		{
			var lib = list[i];
			var dna = lib.dna;
			var idSuffix = lib.bioWell.column + '.' + lib.bioWell.row;
			var tableRow = Doc.element('row.'+idSuffix);
			
			Doc.removeClass(tableRow, 'empty');
			Doc.element('dna.'+idSuffix).innerHTML = Strings.encodeTags(dna.name);
			
			Doc.element('box.'+idSuffix).innerHTML = Strings.encodeTags(protocol.getPlateCoordinate(dna, true));
			Doc.element('panel.'+idSuffix).innerHTML = Strings.encodeTags(dna.MIPS_Panel);
			Doc.addClass('panel.'+idSuffix, MipsColor.getColor(dna.MIPS_Panel));
			
			var form = '';
			if (dna.MIPS_NormalTumor != null) form += '<span>'+Strings.encodeTags(dna.MIPS_NormalTumor)+'</span>';
			if (dna.MIPS_Form != null) form += '<span>'+Strings.encodeTags(dna.MIPS_Form)+'</span>';
			Doc.element('form.'+idSuffix).innerHTML = form;
			Doc.element('volume.'+idSuffix).innerHTML = Numbers.formatNumber(dna.MIPS_PoolVolume, 0, 'µl');
			var remarks = dna.remarks.join('; ');
			if (dna.warnings.length > 0)
			{
				remarks += '<span class="warning">'+dna.warnings.join('; ')+'</span>';
			}
			Doc.element('remarks.'+idSuffix).innerHTML = remarks;
		}
		
		Doc.show('all-protocol');
	}
	
	
	protocol.getPlateCoordinate = function(dna, includePlateName)
	{
		var c = '';
		if (dna.bioWell)
		{
			if (includePlateName) c += dna.bioWell.bioPlate.name + ' ';
			c += '[' + Reggie.wellToAlpha(dna.bioWell.row) + (dna.bioWell.column+1) + ']';
		}
		else
		{
			c = ' ';
		}
		return c;
	}

		
	return protocol;
}();

Doc.onLoad(Protocol.initPage);

var COLUMN_DATA = [];
var WellPainter = function()
{
	var painter = {};
	
	painter.getClassNameForWell = function(well)
	{
		var cls = '';
		if (COLUMN_DATA[well.column]) 
		{
			cls += ' ' + (COLUMN_DATA[well.column].poolClass || '');
			cls += ' ' + (COLUMN_DATA[well.column].groupClass || '');
		}
		if (well.extract) cls += ' ' + MipsColor.getColor(well.extract.dna.MIPS_Panel);
		return cls;
	}
	
	painter.getWellText = function(well)
	{
		var text = '';
		var lib = well.extract;
		if (lib)
		{
			var dna = lib.dna;
			
			if (!dna.nameWithoutPrefix) dna.nameWithoutPrefix = dna.name.replace(/^[a-zA-Z]+_/, '');
			
			text += '<div class="dna">'+Reggie.softLineBreaks(Strings.encodeTags(dna.nameWithoutPrefix))+'</div>';
			text += '<div class="box">'+Strings.encodeTags(Protocol.getPlateCoordinate(dna, false))+'</div>';

			if (dna.MIPS_PoolVolume != null)
			{
				text += '<div class="volume">'+Numbers.formatNumber(dna.MIPS_PoolVolume, 0, 'µl')+'</div>';
			}

			text += '<div class="mips-panel">'+Strings.encodeTags(dna.MIPS_Panel)+'</div>';

			if (dna.MIPS_Form != null || dna.MIPS_NormalTumor != null)
			{
				text += '<div class="mips-form">';
				if (dna.MIPS_NormalTumor != null) text += '<span>'+Strings.encodeTags(dna.MIPS_NormalTumor)+'</span>';
				if (dna.MIPS_Form != null) text += '<span>'+Strings.encodeTags(dna.MIPS_Form)+'</span>';
				text += '</div>';
			}
			
			if (dna.warnings.length > 0)
			{
				well.setWarning(dna.warnings.join('; '));
			}
			
			text += '<div class="remarks">'+ dna.remarks.join('; ') + '</div>';
		}
		return text;
	}

	return painter;
}();

